import _ from 'lodash';
import PouchDB from 'pouchdb';
import { watchDbLegacy } from '../db-admin/health-legacy';
import { openDatabases } from '../db-admin/open';
import { toHex } from '../lib/to-hex';
import changeHandler from './change-handler';

declare global {
  var localDb: any;
  var db: any;
  var dbReady: boolean;
  var currentAccount: any;
  var sync: any;
}

export const initDb = async () => {
  console.log({ caller: 'initDb' });

  if (globalThis.localDb === undefined) {
    globalThis.dbReady = false;

    await openDatabases();

    globalThis.currentAccount = await openDatabases();
    const currentAccount = globalThis.currentAccount;

    if (currentAccount.remoteDbServer) {
      const url = `${currentAccount.remoteDbServer}/userdb-${toHex(
        currentAccount.remoteDbUser
      )}`;

      const remoteDb = new PouchDB(url, {
        auth: {
          username: currentAccount.remoteDbUser,
          password: currentAccount.remoteDbPassword,
        },
        skip_setup: true,
      });
      globalThis.remoteDb = remoteDb;
      globalThis.sync = PouchDB.sync(db, remoteDb, {
        live: true,
        retry: true,
      });
    }

    // if (globalThis.sync) {
    //   globalThis.sync
    //     .on('change', function (info) {
    //       // handle change
    //       console.log({ caller: 'Sync / change', info });
    //     })
    //     .on('paused', function (err) {
    //       // replication paused (e.g. replication up to date, user went offline)
    //       console.log({ caller: 'Sync / paused', err });
    //     })
    //     .on('active', function () {
    //       // replicate resumed (e.g. new changes replicating, user went back online)
    //       console.log({ caller: 'Sync / active' });
    //     })
    //     .on('denied', function (err) {
    //       // a document failed to replicate (e.g. due to permissions)
    //       console.error({ caller: 'Sync / denied', err });
    //     })
    //     .on('complete', function (info) {
    //       // handle complete
    //       console.log({ caller: 'Sync / complete', info });
    //     })
    //     .on('error', function (err) {
    //       // handle error
    //       console.error({ caller: 'Sync / error', err });
    //     });
    // }

    // console.log({ caller: 'initDb / before db.changes' });

    const changes = db
      .changes({ since: 'now', live: true, include_docs: false })
      .on('change', changeHandler)
      .on('complete', (info: any) => {
        console.log({ caller: 'changes / complete', info });
      })
      .on('error', (error: any) => {
        console.log({ caller: 'changes / complete', error });
      });

    const localDbChanges = localDb
      .changes({ since: 'now', live: true, include_docs: false })
      .on('change', changeHandler)
      .on('complete', (info: any) => {
        console.log({ caller: 'localDb changes / complete', info });
      })
      .on('error', (error: any) => {
        console.log({ caller: 'localDb changes / complete', error });
      });

    // console.log({ caller: 'initDb / back from db.changes', changes });

    //  changes.cancel();

    globalThis.dbReady = true;
    watchDbLegacy();
    console.log({ caller: 'initDb / done' });
    return;
  }
};
